%include verbatim.fmt
\begingroup
\let\origtt=\ttfamily
\def\ttfamily#1#2{\origtt}
>%format omega = "\omega"
>|[omega, omega13, omega', omega13']|\par
>%format omega13
>|[omega, omega13, omega', omega13']|\par
>%format omega'
>|[omega, omega13, omega', omega13']|\par
>%format omega13'
>|[omega, omega13, omega', omega13']|
\endgroup
