%include verbatim.fmt
\begingroup
\let\origtt=\ttfamily
\def\ttfamily#1#2{\origtt}

>%format ++         = "\mathbin{\mathbf{+}}"
>%format undefined  = "\Varid{undefined}"
>%format not        = "!"

\endgroup
