%include poly.fmt

%format omega = "\omega"
|[omega, omega13, omega', omega13']|\par
%format omega13
|[omega, omega13, omega', omega13']|\par
%format omega'
|[omega, omega13, omega', omega13']|\par
%format omega13'
|[omega, omega13, omega', omega13']|
