\begin{code}
.
{HOME}/lhs2tex-1.23//
{HOME}/lhs2tex//
{HOME}/lhs2TeX//
{HOME}/.lhs2tex-1.23//
{HOME}/.lhs2tex//
{HOME}/.lhs2TeX//
{LHS2TEX}//
/usr/local/share/lhs2tex-1.23//
/usr/local/share/lhs2tex-1.23//
/usr/local/lib/lhs2tex-1.23//
/usr/share/lhs2tex-1.23//
/usr/lib/lhs2tex-1.23//
/usr/local/share/lhs2tex//
/usr/local/lib/lhs2tex//
/usr/share/lhs2tex//
/usr/lib/lhs2tex//
/usr/local/share/lhs2TeX//
/usr/local/lib/lhs2TeX//
/usr/share/lhs2TeX//
/usr/lib/lhs2TeX//
\end{code}
