%include verbatim.fmt
\begingroup
\let\origtt=\ttfamily
\def\ttfamily#1#2{\origtt}
>%format a_i
>%format a_j
>%format left  = "\leftarrow "
>%format right = "\rightarrow "
>%format a_left
>%format a_right
>%format a_let
>%format a_where
>%format a_x_1
>%format a_x_2
>%format y_1
>%format y_2
>%format a_y_1
>%format a_y_2
>%format a_y1
>%format a_i'
>|[a_i,a_j,a_left,a_right,a_let,a_where,a_x_1,a_x_2,a_y_1,a_y_2,a_y1,a_i']|
\endgroup
