%include poly.fmt

%if False

> module InteractiveHugs where

%endif
%format . = "."
%format forall a = "\forall" a
%options hugs -98

> fix    ::  forall a. (a -> a) -> a
> fix f  =   f (fix f)

This function is of type 
%if "" /= ""
\eval{:t fix},
%else
\textbf{?hugs not found?},
%endif
and |take 10 (fix ('x':))| 
evaluates to
%if "" /= ""
\eval{take 10 (fix ('x':))}.
%else
\textbf{?hugs not found?}.
%endif
